@@ -3,7 +3,7 @@
  * Copyright 2004, Tim Endean <endeant@hotmail.com>
  * Built from the gsm-map dissector Copyright 2004, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-inap-template.c 22049 2007-06-05 17:15:21Z guy $
+ * $Id: packet-inap-template.c 23324 2007-10-31 21:24:16Z etxrab $
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
@@ -33,6 +33,7 @@
 #include <epan/packet.h>
 #include <epan/prefs.h>
 #include <epan/conversation.h>
+#include <epan/oids.h>
 #include "epan/expert.h"
 #include <epan/asn1.h>
 
@@ -585,6 +586,8 @@ void proto_reg_handoff_inap(void) {
     ssn_range = range_copy(global_ssn_range);
 
     range_foreach(ssn_range, range_add_callback);
+
+	oid_add_from_string("Core-INAP-CS1-Codes","0.4.0.1.1.0.3.0");
 }
 
 
