@@ -1,6 +1,6 @@
 /* dumpcap.c
  *
- * $Id: dumpcap.c 24352 2008-02-16 08:44:33Z guy $
+ * $Id: dumpcap.c 29905 2009-09-14 18:53:27Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -96,17 +96,20 @@
 #include "log.h"
 #include "file_util.h"
 
-#ifdef NEED_G_ASCII_STRCASECMP_H
-#include "epan/g_ascii_strcasecmp.h"
-#endif
-
 /*
  * Get information about libpcap format from "wiretap/libpcap.h".
  * XXX - can we just use pcap_open_offline() to read the pipe?
  */
 #include "wiretap/libpcap.h"
 
-/*#define DEBUG_DUMPCAP*/
+/**#define DEBUG_DUMPCAP**/
+/**#define DEBUG_CHILD_DUMPCAP**/
+
+#ifdef DEBUG_CHILD_DUMPCAP
+FILE *debug_log;   /* for logging debug messages to  */
+                   /*  a file if DEBUG_CHILD_DUMPCAP */
+                   /*  is defined                    */
+#endif
 
 static gboolean capture_child = FALSE; /* FALSE: standalone call, TRUE: this is an Wireshark capture child */
 #ifdef _WIN32
@@ -126,7 +129,7 @@ static void capture_loop_stop(void);
  * is interrupted by a signal on UN*X, just go back and try again to
  * read again.
  *
- * On UN*X, we catch SIGUSR1 as a "stop capturing" signal, and, in
+ * On UN*X, we catch SIGINT as a "stop capturing" signal, and, in
  * the signal handler, set a flag to stop capturing; however, without
  * a guarantee of that sort, we can't guarantee that we'll stop capturing
  * if the read will be retried and won't time out if no packets arrive.
@@ -149,7 +152,7 @@ static void capture_loop_stop(void);
  * exit pcap_dispatch() with an indication that no packets have arrived,
  * and will break out of the capture loop at that point.
  *
- * On Windows, we can't send a SIGUSR1 to stop capturing, so none of this
+ * On Windows, we can't send a SIGINT to stop capturing, so none of this
  * applies in any case.
  *
  * XXX - the various BSDs appear to define BSD in <sys/param.h>; we don't
@@ -223,7 +226,7 @@ static const char please_report[] =
     "(This is not a crash; please do not report it as such.)";
 
 /*
- * This needs to be static, so that the SIGUSR1 handler can clear the "go"
+ * This needs to be static, so that the SIGINT handler can clear the "go"
  * flag.
  */
 static loop_data   ld;
@@ -353,9 +356,12 @@ cmdarg_err(const char *fmt, ...)
   va_list ap;
 
   if(capture_child) {
-    /* Print a bare error */
+    gchar *msg;
+    /* Generate a 'special format' message back to parent */
     va_start(ap, fmt);
-    vfprintf(stderr, fmt, ap);
+    msg = g_strdup_vprintf(fmt, ap);
+    sync_pipe_errmsg_to_parent(2, msg, "");
+    g_free(msg);
     va_end(ap);
   } else {
     va_start(ap, fmt);
@@ -375,7 +381,12 @@ cmdarg_err_cont(const char *fmt, ...)
   va_list ap;
 
   if(capture_child) {
-    /* XXX - convert to g_log */
+    gchar *msg;
+    va_start(ap, fmt);
+    msg = g_strdup_vprintf(fmt, ap);
+    sync_pipe_errmsg_to_parent(2, msg, "");
+    g_free(msg);
+    va_end(ap);
   } else {
     va_start(ap, fmt);
     vfprintf(stderr, fmt, ap);
@@ -387,7 +398,7 @@ cmdarg_err_cont(const char *fmt, ...)
 
 #ifdef _WIN32
 static BOOL WINAPI
-capture_cleanup(DWORD dwCtrlType)
+capture_cleanup_handler(DWORD dwCtrlType)
 {
     /* CTRL_C_EVENT is sort of like SIGINT, CTRL_BREAK_EVENT is unique to
        Windows, CTRL_CLOSE_EVENT is sort of like SIGHUP, CTRL_LOGOFF_EVENT
@@ -419,16 +430,16 @@ capture_cleanup(DWORD dwCtrlType)
 }
 #else
 static void
-capture_cleanup(int signum)
+capture_cleanup_handler(int signum _U_)
 {
     /* On UN*X, we cleanly shut down the capture on SIGINT, SIGHUP, and
        SIGTERM.  We assume that if the user wanted it to keep running
        after they logged out, they'd have nohupped it. */
 
-    g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO,
-        "Console: Signal");
-    g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-        "Console: Signal, signal value: %u", signum);
+    /* Note: don't call g_log() in the signal handler: if we happened to be in
+     * g_log() in process context when the signal came in, g_log will detect
+     * the "recursion" and abort.
+     */
 
     capture_loop_stop();
 }
@@ -455,14 +466,20 @@ static void exit_main(int status)
 /*
  * If we were linked with libcap (not libpcap), make sure we have
  * CAP_NET_ADMIN and CAP_NET_RAW, then relinquish our permissions.
+ * (See comment in main() for details)
  */
 
 static void
 #if 0 /* Set to enable capability debugging */
+/* see 'man cap_to_text()' for explanation of output                         */
+/* '='   means 'all= '  ie: no capabilities                                  */
+/* '=ip' means 'all=ip' ie: all capabilities are permissible and inheritable */
+/* ....                                                                      */
 print_caps(char *pfx) {
     cap_t caps = cap_get_proc();
-    fprintf(stderr, "%s: EUID: %d  Capabilities: %s\n", pfx,
-            geteuid(), cap_to_text(caps, NULL));
+    g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
+          "%s: EUID: %d  Capabilities: %s", pfx,
+          geteuid(), cap_to_text(caps, NULL));
     cap_free(caps);
 #else
 print_caps(char *pfx _U_) {
@@ -472,39 +489,63 @@ print_caps(char *pfx _U_) {
 static void
 relinquish_privs_except_capture(void)
 {
-    /* CAP_NET_ADMIN: Promiscuous mode and a truckload of other
+    /* If 'started_with_special_privs' (ie: suid) then enable for
+     *  ourself the  NET_ADMIN and NET_RAW capabilities and then
+     *  drop our suid privileges.
+     *
+     * CAP_NET_ADMIN: Promiscuous mode and a truckload of other
      *                stuff we don't need (and shouldn't have).
      * CAP_NET_RAW:   Packet capture (raw sockets).
      */
-    cap_value_t cap_list[2] = { CAP_NET_ADMIN, CAP_NET_RAW };
-    cap_t caps = cap_init();
-    int cl_len = sizeof(cap_list) / sizeof(cap_value_t);
 
     if (started_with_special_privs()) {
+        cap_value_t cap_list[2] = { CAP_NET_ADMIN, CAP_NET_RAW };
+        int cl_len = sizeof(cap_list) / sizeof(cap_value_t);
+
+        cap_t caps = cap_init();    /* all capabilities initialized to off */
+
         print_caps("Pre drop, pre set");
+
         if (prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0) == -1) {
-            perror("prctl()");
+            cmdarg_err("prctl() fail return: %s", strerror(errno));
         }
 
         cap_set_flag(caps, CAP_PERMITTED,   cl_len, cap_list, CAP_SET);
         cap_set_flag(caps, CAP_INHERITABLE, cl_len, cap_list, CAP_SET);
 
         if (cap_set_proc(caps)) {
-            perror("capset()");
+            cmdarg_err("cap_set_proc() fail return: %s", strerror(errno));
         }
         print_caps("Pre drop, post set");
+
+        relinquish_special_privs_perm();
+
+        print_caps("Post drop, pre set");
+        cap_set_flag(caps, CAP_EFFECTIVE,   cl_len, cap_list, CAP_SET);
+        if (cap_set_proc(caps)) {
+            cmdarg_err("cap_set_proc() fail return: %s", strerror(errno));
+        }
+        print_caps("Post drop, post set");
+
+        cap_free(caps);
     }
+}
 
-    relinquish_special_privs_perm();
 
-    print_caps("Post drop, pre set");
-    cap_set_flag(caps, CAP_EFFECTIVE,   cl_len, cap_list, CAP_SET);
+static void
+relinquish_all_capabilities()
+{
+    /* Drop any and all capabilities this process may have.            */
+    /* Allowed whether or not process has any privileges.              */
+    cap_t caps = cap_init();    /* all capabilities initialized to off */
+    print_caps("Pre-clear");
     if (cap_set_proc(caps)) {
-        perror("capset()");
+        cmdarg_err("cap_set_proc() fail return: %s", strerror(errno));
     }
-    print_caps("Post drop, post set");
+    print_caps("Post-clear");
     cap_free(caps);
 }
+
 #endif /* HAVE_LIBCAP */
 
 /* Take care of byte order in the libpcap headers read from pipes.
@@ -1069,8 +1110,15 @@ capture_loop_open_input(capture_options *capture_opts, loop_data *ld,
 		       open_err_str);
 #endif
 
+/* If not using libcap: we now can now set euid/egid to ruid/rgid         */
+/*  to remove any suid privileges.                                        */
+/* If using libcap: we can now remove NET_RAW and NET_ADMIN capabilities  */
+/*  (euid/egid have already previously been set to ruid/rgid.             */
+/* (See comment in main() for details)                                    */
 #ifndef HAVE_LIBCAP
   relinquish_special_privs_perm();
+#else
+  relinquish_all_capabilities();
 #endif
 
   if (ld->pcap_h != NULL) {
@@ -1642,13 +1690,6 @@ capture_loop_open_output(capture_options *capture_opts, int *save_file_fd,
 }
 
 
-static void
-capture_loop_stop_signal_handler(int signo _U_)
-{
-  g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO, "Signal: Stop capture");
-  capture_loop_stop();
-}
-
 #ifdef _WIN32
 #define TIME_GET() GetTickCount()
 #else
@@ -1660,9 +1701,6 @@ capture_loop_stop_signal_handler(int signo _U_)
 static gboolean
 capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct pcap_stat *stats)
 {
-#ifndef _WIN32
-  struct sigaction act;
-#endif
   time_t      upd_time, cur_time;
   time_t      start_time;
   int         err_close;
@@ -1704,22 +1742,6 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   /* We haven't yet gotten the capture statistics. */
   *stats_known      = FALSE;
 
-#ifndef _WIN32
-  /*
-   * Catch SIGUSR1, so that we exit cleanly if the parent process
-   * kills us with it due to the user selecting "Capture->Stop".
-   */
-  act.sa_handler = capture_loop_stop_signal_handler;
-  /*
-   * Arrange that system calls not get restarted, because when
-   * our signal handler returns we don't want to restart
-   * a call that was waiting for packets to arrive.
-   */
-  act.sa_flags = 0;
-  sigemptyset(&act.sa_mask);
-  sigaction(SIGUSR1, &act, NULL);
-#endif /* _WIN32 */
-
   g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO, "Capture loop starting ...");
   capture_opts_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG, capture_opts);
 
@@ -2148,13 +2170,6 @@ capture_loop_packet_cb(u_char *user, const struct pcap_pkthdr *phdr,
   loop_data *ld = (void *) user;
   int err;
 
-  /* if the user told us to stop after x packets, do we already have enough? */
-  if ((ld->packet_max > 0) && (ld->packet_count >= ld->packet_max))
-  {
-     ld->go = FALSE;
-     return;
-  }
-
   /* We may be called multiple times from pcap_dispatch(); if we've set
      the "stop capturing" flag, ignore this packet, as we're not
      supposed to be saving any more packets. */
@@ -2168,8 +2183,14 @@ capture_loop_packet_cb(u_char *user, const struct pcap_pkthdr *phdr,
     if (!libpcap_write_packet(ld->pdh, phdr, pd, &ld->bytes_written, &err)) {
       ld->go = FALSE;
       ld->err = err;
-    } else
+    } else {
       ld->packet_count++;
+      /* if the user told us to stop after x packets, do we already have enough? */
+      if ((ld->packet_max > 0) && (ld->packet_count >= ld->packet_max))
+      {
+        ld->go = FALSE;
+      }
+    }
   }
 }
 
@@ -2198,6 +2219,7 @@ main(int argc, char *argv[])
   gboolean             machine_readable = FALSE;
   gboolean             print_statistics = FALSE;
   int                  status, run_once_args = 0;
+  gint                 i;
 
 #ifdef HAVE_PCAP_REMOTE
 #define OPTSTRING_INIT "a:A:b:c:Df:hi:Lm:MprSs:uvw:y:Z:"
@@ -2214,6 +2236,67 @@ main(int argc, char *argv[])
   char optstring[sizeof(OPTSTRING_INIT) + sizeof(OPTSTRING_WIN32) - 1] =
     OPTSTRING_INIT OPTSTRING_WIN32;
 
+#ifdef DEBUG_CHILD_DUMPCAP
+  if ((debug_log = fopen("dumpcap_debug_log.tmp","w")) == NULL) {
+          fprintf (stderr, "Unable to open debug log file !\n");
+          exit (1);
+  }
+#endif
+
+  /* Determine if dumpcap is being requested to run in a special       */
+  /* capture_child mode by going thru the command line args to see if  */
+  /* a -Z is present. (-Z is a hidden option).                         */
+  /* The primary result of running in capture_child mode is that       */
+  /* all messages sent out on stderr are in a special type/len/string  */
+  /* format to allow message processing by type.                       */
+  /* These messages include various 'status' messages which are sent   */
+  /* when an actual capture is in progress. Capture_child mode         */
+  /* would normally be requested by a parent process which invokes     */
+  /* dumpcap and obtains dumpcap stderr output via a pipe to which     */
+  /* dumpcap stderr has been redirected.                               */
+  /* Capture_child mode needs to be determined immediately upon        */
+  /* startup so that any messages generated by dumpcap in this mode    */
+  /* (eg: during initialization) will be formatted properly.           */
+
+  for (i=1; i<argc; i++) {
+    if (strcmp("-Z", argv[i]) == 0) {
+      capture_child = TRUE;
+#ifdef _WIN32
+      /* set output pipe to binary mode, to avoid ugly text conversions */
+      _setmode(2, O_BINARY);
+#endif
+    }
+  }
+
+  /* The default_log_handler will use stdout, which makes trouble in   */
+  /* capture child mode, as it uses stdout for it's sync_pipe.         */
+  /* So: the filtering is done in the console_log_handler and not here.*/
+  /* We set the log handlers right up front to make sure that any log  */
+  /* messages when running as child will be sent back to the parent    */
+  /* with the correct format.                                          */
+
+  log_flags =
+                    G_LOG_LEVEL_ERROR|
+                    G_LOG_LEVEL_CRITICAL|
+                    G_LOG_LEVEL_WARNING|
+                    G_LOG_LEVEL_MESSAGE|
+                    G_LOG_LEVEL_INFO|
+                    G_LOG_LEVEL_DEBUG|
+                    G_LOG_FLAG_FATAL|G_LOG_FLAG_RECURSION;
+
+  g_log_set_handler(NULL,
+                    log_flags,
+                    console_log_handler, NULL /* user_data */);
+  g_log_set_handler(LOG_DOMAIN_MAIN,
+                    log_flags,
+                    console_log_handler, NULL /* user_data */);
+  g_log_set_handler(LOG_DOMAIN_CAPTURE,
+                    log_flags,
+                    console_log_handler, NULL /* user_data */);
+  g_log_set_handler(LOG_DOMAIN_CAPTURE_CHILD,
+                    log_flags,
+                    console_log_handler, NULL /* user_data */);
+
 #ifdef _WIN32
   /* Load wpcap if possible. Do this before collecting the run-time version information */
   load_wpcap();
@@ -2226,50 +2309,108 @@ main(int argc, char *argv[])
   WSAStartup( MAKEWORD( 1, 1 ), &wsaData );
 
   /* Set handler for Ctrl+C key */
-  SetConsoleCtrlHandler(capture_cleanup, TRUE);
+  SetConsoleCtrlHandler(capture_cleanup_handler, TRUE);
 #else
   /* Catch SIGINT and SIGTERM and, if we get either of them, clean up
      and exit. */
-  action.sa_handler = capture_cleanup;
+  action.sa_handler = capture_cleanup_handler;
+  /*
+   * Arrange that system calls not get restarted, because when
+   * our signal handler returns we don't want to restart
+   * a call that was waiting for packets to arrive.
+   */
   action.sa_flags = 0;
   sigemptyset(&action.sa_mask);
   sigaction(SIGTERM, &action, NULL);
   sigaction(SIGINT, &action, NULL);
+  sigaction(SIGPIPE, &action, NULL);
   sigaction(SIGHUP, NULL, &oldaction);
   if (oldaction.sa_handler == SIG_DFL)
     sigaction(SIGHUP, &action, NULL);
 #endif  /* _WIN32 */
 
+  /* ----------------------------------------------------------------- */
+  /* Privilege and capability handling                                 */
+  /* Cases:                                                            */
+  /* 1. Running not as root or suid root; no special capabilities.     */
+  /*    Action: none                                                   */
+  /*                                                                   */
+  /* 2. Running logged in as root (euid=0; ruid=0); Not using libcap.  */
+  /*    Action: none                                                   */
+  /*                                                                   */
+  /* 3. Running logged in as root (euid=0; ruid=0). Using libcap.      */
+  /*    Action:                                                        */
+  /*      - Near start of program: Enable NET_RAW and NET_ADMIN        */
+  /*        capabilities; Drop all other capabilities;                 */
+  /*      - If not -w  (ie: doing -S or -D, etc) run to completion;    */
+  /*        else: after  pcap_open_live() in capture_loop_open_input() */
+  /*         drop all capabilities (NET_RAW and NET_ADMIN);            */
+  /*         (Note: this means that the process, although logged in    */
+  /*          as root, does not have various permissions such as the   */
+  /*          ability to bypass file access permissions).              */
+  /*      XXX: Should we just leave capabilities alone in this case    */
+  /*          so that user gets expected effect that root can do       */
+  /*          anything ??                                              */
+  /*                                                                   */
+  /* 4. Running as suid root (euid=0, ruid=n); Not using libcap.       */
+  /*    Action:                                                        */
+  /*      - If not -w  (ie: doing -S or -D, etc) run to completion;    */
+  /*        else: after  pcap_open_live() in capture_loop_open_input() */
+  /*         drop suid root (set euid=ruid).(ie: keep suid until after */
+  /*         pcap_open_live).                                          */
+  /*                                                                   */
+  /* 5. Running as suid root (euid=0, ruid=n); Using libcap.           */
+  /*    Action:                                                        */
+  /*      - Near start of program: Enable NET_RAW and NET_ADMIN        */
+  /*        capabilities; Drop all other capabilities;                 */
+  /*        Drop suid privileges (euid=ruid);                          */
+  /*      - If not -w  (ie: doing -S or -D, etc) run to completion;    */
+  /*        else: after  pcap_open_live() in capture_loop_open_input() */
+  /*         drop all capabilities (NET_RAW and NET_ADMIN).            */
+  /*                                                                   */
+  /*      XXX: For some Linux versions/distros with capabilities       */
+  /*        a 'normal' process with any capabilities cannot be         */
+  /*        'killed' (signaled) from another (same uid) non-privileged */
+  /*        process.                                                   */
+  /*        For example: If (non-suid) Wireshark forks a               */
+  /*        child suid dumpcap which acts as described here (case 5),  */
+  /*        Wireshark will be unable to kill (signal) the child        */
+  /*        dumpcap process until the capabilities have been dropped   */
+  /*        (after pcap_open_live()).                                  */
+  /*        This behaviour will apparently be changed in the kernel    */
+  /*        to allow the kill (signal) in this case.                   */
+  /*        See the following for details:                             */
+  /*           http://www.mail-archive.com/  [wrapped]                 */
+  /*             linux-security-module@vger.kernel.org/msg02913.html   */
+  /*                                                                   */
+  /*        It is therefore conceivable that if dumpcap somehow hangs  */
+  /*        in pcap_open_live or before that wireshark will not        */
+  /*        be able to stop dumpcap using a signal (INT, TERM, etc).  */
+  /*        In this case, exiting wireshark will kill the child        */
+  /*        dumpcap process.                                           */
+  /*                                                                   */
+  /* 6. Not root or suid root; Running with NET_RAW & NET_ADMIN        */
+  /*     capabilities; Using libcap.  Note: capset cmd (which see)     */
+  /*     used to assign capabilities to file.                          */
+  /*    Action:                                                        */
+  /*      - If not -w  (ie: doing -S or -D, etc) run to completion;    */
+  /*        else: after  pcap_open_live() in capture_loop_open_input() */
+  /*         drop all capabilities (NET_RAW and NET_ADMIN)             */
+  /*                                                                   */
+  /* ToDo: -S (stats) should drop privileges/capabilities when no      */
+  /*       longer required (similar to capture).                        */
+  /*                                                                   */
+  /* ----------------------------------------------------------------- */
+
   get_credential_info();
+
 #ifdef HAVE_LIBCAP
+  /* If 'started with special privileges' (and using libcap)  */
+  /*   Set to keep only NET_RAW and NET_ADMIN capabilities;   */
+  /*   Set euid/egid = ruid/rgid to remove suid privileges    */
   relinquish_privs_except_capture();
 #endif
 
-  /* the default_log_handler will use stdout, which makes trouble in */
-  /* capture child mode, as it uses stdout for it's sync_pipe */
-  /* so do the filtering in the console_log_handler and not here */
-  log_flags =
-		    G_LOG_LEVEL_ERROR|
-		    G_LOG_LEVEL_CRITICAL|
-		    G_LOG_LEVEL_WARNING|
-		    G_LOG_LEVEL_MESSAGE|
-		    G_LOG_LEVEL_INFO|
-		    G_LOG_LEVEL_DEBUG|
-		    G_LOG_FLAG_FATAL|G_LOG_FLAG_RECURSION;
-
-  g_log_set_handler(NULL,
-		    log_flags,
-		    console_log_handler, NULL /* user_data */);
-  g_log_set_handler(LOG_DOMAIN_MAIN,
-		    log_flags,
-		    console_log_handler, NULL /* user_data */);
-  g_log_set_handler(LOG_DOMAIN_CAPTURE,
-		    log_flags,
-            console_log_handler, NULL /* user_data */);
-  g_log_set_handler(LOG_DOMAIN_CAPTURE_CHILD,
-		    log_flags,
-            console_log_handler, NULL /* user_data */);
-
   /* Set the initial values in the capture_opts. This might be overwritten
      by the command line parameters. */
   capture_opts_init(capture_opts, NULL);
@@ -2329,34 +2470,33 @@ main(int argc, char *argv[])
 #endif /* _WIN32 */
         status = capture_opts_add_opt(capture_opts, opt, optarg, &start_capture);
         if(status != 0) {
-            exit_main(status);
+          exit_main(status);
         }
         break;
       /*** hidden option: Wireshark child mode (using binary output messages) ***/
       case 'Z':
-          capture_child = TRUE;
+        capture_child = TRUE;
 #ifdef _WIN32
-          /* set output pipe to binary mode, to avoid ugly text conversions */
-	  _setmode(2, O_BINARY);
-          /*
-           * optarg = the control ID, aka the PPID, currently used for the
-           * signal pipe name.
-           */
-          if (strcmp(optarg, SIGNAL_PIPE_CTRL_ID_NONE) != 0) {
-              sig_pipe_name = g_strdup_printf(SIGNAL_PIPE_FORMAT,
-                  optarg);
-              sig_pipe_handle = CreateFile(utf_8to16(sig_pipe_name),
-                  GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
+        /* set output pipe to binary mode, to avoid ugly text conversions */
+	_setmode(2, O_BINARY);
+        /*
+         * optarg = the control ID, aka the PPID, currently used for the
+         * signal pipe name.
+         */
+        if (strcmp(optarg, SIGNAL_PIPE_CTRL_ID_NONE) != 0) {
+          sig_pipe_name = g_strdup_printf(SIGNAL_PIPE_FORMAT, optarg);
+          sig_pipe_handle = CreateFile(utf_8to16(sig_pipe_name),
+              GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
 
-              if (sig_pipe_handle == INVALID_HANDLE_VALUE) {
-                  g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO,
-                      "Signal pipe: Unable to open %s.  Dead parent?",
-                      sig_pipe_name);
-                  exit_main(1);
-              }
+          if (sig_pipe_handle == INVALID_HANDLE_VALUE) {
+            g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO,
+                  "Signal pipe: Unable to open %s.  Dead parent?",
+                  sig_pipe_name);
+            exit_main(1);
           }
+        }
 #endif
-          break;
+        break;
 
       /*** all non capture option specific ***/
       case 'D':        /* Print a list of capture devices and exit */
@@ -2384,8 +2524,8 @@ main(int argc, char *argv[])
   argc -= optind;
   argv += optind;
   if (argc >= 1) {
-      /* user specified file name as regular command-line argument */
-      /* XXX - use it as the capture file name (or something else)? */
+    /* user specified file name as regular command-line argument */
+    /* XXX - use it as the capture file name (or something else)? */
     argc--;
     argv++;
   }
@@ -2436,7 +2576,7 @@ main(int argc, char *argv[])
   }
 
   if (capture_opts_trim_iface(capture_opts, NULL) == FALSE) {
-	cmdarg_err("No capture interfaces available (maybe lack of privileges?).");
+    /* cmdarg_err() already called .... */
     exit_main(1);
   }
 
@@ -2461,11 +2601,11 @@ main(int argc, char *argv[])
   /* Now start the capture. */
 
   if(capture_loop_start(capture_opts, &stats_known, &stats) == TRUE) {
-      /* capture ok */
-      exit_main(0);
+    /* capture ok */
+    exit_main(0);
   } else {
-      /* capture failed */
-      exit_main(1);
+    /* capture failed */
+    exit_main(1);
   }
 }
 
@@ -2475,13 +2615,13 @@ console_log_handler(const char *log_domain, GLogLevelFlags log_level,
 		    const char *message, gpointer user_data _U_)
 {
   time_t curr;
-  struct tm *today;
+  struct tm  *today;
   const char *level;
-
+  gchar      *msg;
 
   /* ignore log message, if log_level isn't interesting */
   if( !(log_level & G_LOG_LEVEL_MASK & ~(G_LOG_LEVEL_DEBUG|G_LOG_LEVEL_INFO))) {
-#ifndef DEBUG_DUMPCAP
+#if !defined(DEBUG_DUMPCAP) && !defined(DEBUG_CHILD_DUMPCAP)
     return;
 #endif
   }
@@ -2515,19 +2655,43 @@ console_log_handler(const char *log_domain, GLogLevelFlags log_level,
     g_assert_not_reached();
   }
 
-  /* don't use printf (stdout), in child mode we're using stdout for the sync_pipe */
+  /* Generate the output message                                  */
   if(log_level & G_LOG_LEVEL_MESSAGE) {
     /* normal user messages without additional infos */
-    fprintf(stderr, "%s\n", message);
-    fflush(stderr);
+    msg =  g_strdup_printf("%s\n", message);
   } else {
     /* info/debug messages with additional infos */
-    fprintf(stderr, "%02u:%02u:%02u %8s %s %s\n",
+    msg = g_strdup_printf("%02u:%02u:%02u %8s %s %s\n",
             today->tm_hour, today->tm_min, today->tm_sec,
             log_domain != NULL ? log_domain : "",
             level, message);
+  }
+
+  /* DEBUG & INFO msgs (if we're debugging today)                 */
+#if defined(DEBUG_DUMPCAP) || defined(DEBUG_CHILD_DUMPCAP)
+  if( !(log_level & G_LOG_LEVEL_MASK & ~(G_LOG_LEVEL_DEBUG|G_LOG_LEVEL_INFO))) {
+#ifdef DEBUG_DUMPCAP
+    fprintf(stderr, "%s", msg);
+    fflush(stderr);
+#endif
+#ifdef DEBUG_CHILD_DUMPCAP
+    fprintf(debug_log, "%s", msg);
+    fflush(debug_log);
+#endif
+    g_free(msg);
+    return;
+  }
+#endif
+
+  /* ERROR, CRITICAL, WARNING, MESSAGE messages goto stderr or    */
+  /*  to parent especially formatted if dumpcap running as child. */
+  if (capture_child) {
+    sync_pipe_errmsg_to_parent(2, msg, "");
+  } else {
+    fprintf(stderr, "%s", msg);
     fflush(stderr);
   }
+  g_free(msg);
 }
 
 
