@@ -1,7 +1,7 @@
 /* capture_if_dlg.c
  * Routines for the capture interface dialog
  *
- * $Id: capture_if_dlg.c 20116 2006-12-11 19:43:19Z ulfl $
+ * $Id: capture_if_dlg.c 29702 2009-09-04 21:26:50Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -48,6 +48,7 @@
 #include "capture_dlg.h"
 #include "capture_if_details_dlg.h"
 #include "capture_errs.h"
+#include "capture_ui_utils.h"
 #include "recent.h"
 #include <epan/prefs.h>
 
@@ -80,6 +81,10 @@
 #include "airpcap_dlg.h"
 #endif
 
+#ifdef NEED_G_ASCII_STRCASECMP_H
+#include "../epan/g_ascii_strcasecmp.h"
+#endif
+
 /*
  * Keep a static pointer to the current "Capture Interfaces" window, if
  * any, so that if somebody tries to do "Capture:Start" while there's
@@ -107,7 +112,6 @@ GList           *if_list;
 
 /* the "runtime" data of one interface */
 typedef struct if_dlg_data_s {
-    pcap_t      *pch;
     GtkWidget   *device_lb;
     GtkWidget   *descr_lb;
     GtkWidget   *ip_lb;
@@ -120,10 +124,9 @@ typedef struct if_dlg_data_s {
 #endif
     guint32     last_packets;
     gchar       *device;
+    if_info_t   if_info;
 } if_dlg_data_t;
 
-void update_if(if_dlg_data_t *if_dlg_data);
-
 
 /* start capture button was pressed */
 static void
@@ -132,14 +135,17 @@ capture_do_cb(GtkWidget *capture_bt _U_, gpointer if_data)
   if_dlg_data_t *if_dlg_data = if_data;
 
 #ifdef HAVE_AIRPCAP
-  airpcap_if_active = get_airpcap_if_from_description(airpcap_if_list, GTK_LABEL(if_dlg_data->descr_lb)->label);
+  airpcap_if_active = get_airpcap_if_from_name(airpcap_if_list, if_dlg_data->if_info.name);
   airpcap_if_selected = airpcap_if_active;
 #endif
 
   if (capture_opts->iface)
     g_free(capture_opts->iface);
+  if (capture_opts->iface_descr)
+    g_free(capture_opts->iface_descr);
 
   capture_opts->iface = g_strdup(if_dlg_data->device);
+  capture_opts->iface_descr = get_interface_descriptive_name(capture_opts->iface);
 
   /* XXX - remove this? */
   if (capture_opts->save_file) {
@@ -162,8 +168,11 @@ capture_prepare_cb(GtkWidget *prepare_bt _U_, gpointer if_data)
 
   if (capture_opts->iface)
     g_free(capture_opts->iface);
+  if (capture_opts->iface_descr)
+    g_free(capture_opts->iface_descr);
 
   capture_opts->iface = g_strdup(if_dlg_data->device);
+  capture_opts->iface_descr = get_interface_descriptive_name(capture_opts->iface);
 
   /* stop capturing from all interfaces, we are going to do real work now ... */
   window_destroy(cap_if_w);
@@ -184,73 +193,27 @@ capture_details_cb(GtkWidget *details_bt _U_, gpointer if_data)
 }
 #endif
 
-
-/* open a single interface */
-static void
-open_if(gchar *name, if_dlg_data_t *if_dlg_data)
-{
-  gchar       open_err_str[CAPTURE_PCAP_ERRBUF_SIZE];
-
-  /*
-   * XXX - on systems with BPF, the number of BPF devices limits the
-   * number of devices on which you can capture simultaneously.
-   *
-   * This means that
-   *
-   *	1) this might fail if you run out of BPF devices
-   *
-   * and
-   *
-   *	2) opening every interface could leave too few BPF devices
-   *	   for *other* programs.
-   *
-   * It also means the system could end up getting a lot of traffic
-   * that it has to pass through the networking stack and capture
-   * mechanism, so opening all the devices and presenting packet
-   * counts might not always be a good idea.
-   */
-  if_dlg_data->pch = pcap_open_live(name,
-		       MIN_PACKET_SIZE,
-		       capture_opts->promisc_mode, CAP_READ_TIMEOUT,
-		       open_err_str);
-
-  if (if_dlg_data->pch != NULL) {
-    update_if(if_dlg_data);
-  } else {
-    printf("open_if: %s\n", open_err_str);
-    gtk_label_set_text(GTK_LABEL(if_dlg_data->curr_lb), "error");
-    gtk_label_set_text(GTK_LABEL(if_dlg_data->last_lb), "error");
-  }
-}
-
 /* update a single interface */
 void
-update_if(if_dlg_data_t *if_dlg_data)
+update_if(if_dlg_data_t *if_dlg_data, if_stat_cache_t *sc)
 {
   struct pcap_stat stats;
   gchar *str;
   guint diff;
 
 
-  /* pcap_stats() stats values differ on libpcap and winpcap!
-   * libpcap: returns the number of packets since pcap_open_live
-   * winpcap: returns the number of packets since the last pcap_stats call
-   * XXX - if that's true, that's a bug, and should be fixed; "pcap_stats()"
-   * is supposed to work the same way on all platforms, including Windows.
-   * Note that the WinPcap 3.0 documentation says "The values represent
-   * packet statistics from the start of the run to the time of the call."
-   * (Note also that some versions of libpcap, on some versions of UN*X,
-   * have the same bug.)
+  /*
+   * Note that some versions of libpcap, on some versions of UN*X,
+   * pcap_stats() returns the number of packets since the last 
+   * pcap_stats call.
+   *
+   * That's a bug, and should be fixed; "pcap_stats()" is supposed 
+   * to work the same way on all platforms.
    */
-  if (if_dlg_data->pch) {
-    if(pcap_stats(if_dlg_data->pch, &stats) >= 0) {
-#ifdef _WIN32
+  if (sc) {
+    if(capture_stats(sc, if_dlg_data->device, &stats)) {
       diff = stats.ps_recv - if_dlg_data->last_packets;
       if_dlg_data->last_packets = stats.ps_recv;
-#else
-      diff = stats.ps_recv;
-      if_dlg_data->last_packets = stats.ps_recv + if_dlg_data->last_packets;
-#endif
 
       str = g_strdup_printf("%u", if_dlg_data->last_packets);
       gtk_label_set_text(GTK_LABEL(if_dlg_data->curr_lb), str);
@@ -268,31 +231,20 @@ update_if(if_dlg_data_t *if_dlg_data)
   }
 }
 
-
-/* close a single interface */
-static void
-close_if(if_dlg_data_t *if_dlg_data)
-{
-    if(if_dlg_data->pch)
-        pcap_close(if_dlg_data->pch);
-}
-
-
-
 /* update all interfaces */
 static gboolean
 update_all(gpointer data)
 {
     GList *curr;
     int ifs;
-
+    if_stat_cache_t *sc = data;
 
     if(!cap_if_w) {
         return FALSE;
     }
 
-    for(ifs = 0; (curr = g_list_nth(data, ifs)); ifs++) {
-        update_if(curr->data);
+    for(ifs = 0; (curr = g_list_nth(if_data, ifs)); ifs++) {
+        update_if(curr->data, sc);
     }
 
     return TRUE;
@@ -321,17 +273,15 @@ set_capture_if_dialog_for_capture_in_progress(gboolean capture_in_progress)
 
 /* the window was closed, cleanup things */
 static void
-capture_if_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
+capture_if_destroy_cb(GtkWidget *win _U_, gpointer user_data)
 {
     GList *curr;
     int ifs;
+    if_stat_cache_t *sc = user_data;
 
     gtk_timeout_remove(timer_id);
 
     for(ifs = 0; (curr = g_list_nth(if_data, ifs)); ifs++) {
-        if_dlg_data_t *if_dlg_data = curr->data;
-
-        close_if(if_dlg_data);
         g_free(curr->data);
     }
 
@@ -342,11 +292,14 @@ capture_if_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
     /* Note that we no longer have a "Capture Options" dialog box. */
     cap_if_w = NULL;
 
+    capture_stat_stop(sc);
+
 #ifdef HAVE_AIRPCAP
-	airpcap_set_toolbar_stop_capture(airpcap_if_active);
+    airpcap_set_toolbar_stop_capture(airpcap_if_active);
 #endif
 }
 
+#if 0
 GtkWidget*
 combo_channel_new(void)
 {
@@ -375,7 +328,7 @@ combo_channel_new(void)
 	  popdown = g_list_append(popdown, "14");
 
       gtk_combo_set_popdown_strings( GTK_COMBO(channel_cb), popdown) ;
-
+      g_list_free(popdown);
 	  #if GTK_MAJOR_VERSION < 2
 	  gtk_widget_set_usize( GTK_WIDGET(channel_cb),
                                   45,
@@ -389,6 +342,21 @@ combo_channel_new(void)
 
 	  return channel_cb;
 }
+#endif
+
+/*
+ * Sorts the Interface List in alphabetical order
+ */
+gint if_list_comparator_alph (const void *first_arg, const void *second_arg){
+  const if_info_t *first = first_arg, *second = second_arg;
+
+  if (first != NULL && first->description != NULL &&
+      second != NULL && second->description != NULL) {
+    return g_ascii_strcasecmp(first->description, second->description);
+  } else {
+    return 0;
+  }
+}
 
 /* start getting capture stats from all interfaces */
 void
@@ -412,8 +380,7 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
 #endif
   GtkTooltips   *tooltips;
   int           err;
-  char          err_str[CAPTURE_PCAP_ERRBUF_SIZE];
-  gchar         *cant_get_if_list_errstr;
+  gchar         *err_str;
   GtkRequisition requisition;
   int           row, height;
   if_dlg_data_t *if_dlg_data;
@@ -424,6 +391,8 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
   if_addr_t     *ip_addr;
   GString       *if_tool_str = g_string_new("");
   gchar         *tmp_str;
+  gchar         *user_descr;
+  if_stat_cache_t *sc;
 
   if (cap_if_w != NULL) {
     /* There's already a "Capture Interfaces" dialog box; reactivate it. */
@@ -444,46 +413,42 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
 #endif
 
   /* LOAD THE INTERFACES */
-  if_list = get_interface_list(&err, err_str);
+  if_list = capture_interface_list(&err, &err_str);
+  if_list = g_list_sort (if_list, if_list_comparator_alph);
   if (if_list == NULL && err == CANT_GET_INTERFACE_LIST) {
-    cant_get_if_list_errstr = cant_get_if_list_error_message(err_str);
-    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s",
-                  cant_get_if_list_errstr);
-    g_free(cant_get_if_list_errstr);
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", err_str);
+    g_free(err_str);
     return;
   }
 
 #ifdef HAVE_AIRPCAP
   /* LOAD AIRPCAP INTERFACES */
-	/* load the airpcap interfaces */
-	airpcap_if_list = get_airpcap_interface_list(&err, err_str);
-	if(airpcap_if_list == NULL) airpcap_if_active = airpcap_if_selected = NULL;
+  airpcap_if_list = get_airpcap_interface_list(&err, &err_str);
+  if (airpcap_if_list == NULL)
+    airpcap_if_active = airpcap_if_selected = NULL;
 
-	decryption_cm = OBJECT_GET_DATA(airpcap_tb,AIRPCAP_TOOLBAR_DECRYPTION_KEY);
-	update_decryption_mode_list(decryption_cm);
+  decryption_cm = OBJECT_GET_DATA(airpcap_tb,AIRPCAP_TOOLBAR_DECRYPTION_KEY);
+  update_decryption_mode_list(decryption_cm);
 
-	if (airpcap_if_list == NULL && err == CANT_GET_AIRPCAP_INTERFACE_LIST) {
-	cant_get_if_list_errstr = cant_get_airpcap_if_list_error_message(err_str);
-	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s",
-				  cant_get_if_list_errstr);
-	g_free(cant_get_if_list_errstr);
-	}
+  if (airpcap_if_list == NULL && err == CANT_GET_AIRPCAP_INTERFACE_LIST) {
+#if 0
+    /* XXX - Do we need to show an error here? */
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", err_str);
+#endif
+    g_free(err_str);
+  }
 
-	/* If no airpcap interface is present, gray everything */
-	if(airpcap_if_active == NULL)
-		{
-		if(airpcap_if_list == NULL)
-			{
-			/*No airpcap device found */
-			airpcap_enable_toolbar_widgets(airpcap_tb,FALSE);
-			}
-		else
-			{
-			/* default adapter is not airpcap... or is airpcap but is not found*/
-			airpcap_set_toolbar_stop_capture(airpcap_if_active);
-			airpcap_enable_toolbar_widgets(airpcap_tb,FALSE);
-			}
-		}
+  /* If no airpcap interface is present, gray everything */
+  if (airpcap_if_active == NULL) {
+    if (airpcap_if_list == NULL) {
+      /*No airpcap device found */
+      airpcap_enable_toolbar_widgets(airpcap_tb,FALSE);
+    } else {
+      /* default adapter is not airpcap... or is airpcap but is not found*/
+      airpcap_set_toolbar_stop_capture(airpcap_if_active);
+      airpcap_enable_toolbar_widgets(airpcap_tb,FALSE);
+    }
+  }
 
   airpcap_set_toolbar_start_capture(airpcap_if_active);
 #endif
@@ -557,14 +522,25 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
   gtk_widget_size_request(stop_bt, &requisition);
   height += requisition.height + 15;
 
+  /* Start gathering statistics (using dumpcap) */
+  sc = capture_stat_start(if_list);
+
+  /* List the interfaces */
   for(ifs = 0; (curr = g_list_nth(if_list, ifs)); ifs++) {
       g_string_assign(if_tool_str, "");
       if_info = curr->data;
+
+      /* Continue if capture device is hidden */
+      if (prefs_is_capture_device_hidden(if_info->name)) {
+          continue;
+      }
+
       if_dlg_data = g_malloc0(sizeof(if_dlg_data_t));
+      if_dlg_data->if_info = *if_info;
 
       /* Kind of adaptor (icon) */
 #ifdef HAVE_AIRPCAP
-      if(get_airpcap_if_from_description(airpcap_if_list,if_info->description) != NULL)
+      if(get_airpcap_if_from_name(airpcap_if_list,if_info->name) != NULL)
         icon = xpm_to_widget(capture_airpcap_16_xpm);
       else
         icon = xpm_to_widget(capture_ethernet_16_xpm);
@@ -586,10 +562,16 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
       g_string_append(if_tool_str, "\n");
 
       /* description */
-      if (if_info->description != NULL)
-        if_dlg_data->descr_lb = gtk_label_new(if_info->description);
-      else
-        if_dlg_data->descr_lb = gtk_label_new("");
+      user_descr = capture_dev_user_descr_find(if_info->name);
+      if (user_descr) {
+        if_dlg_data->descr_lb = gtk_label_new(user_descr);
+        g_free (user_descr);
+      } else {
+        if (if_info->description)
+          if_dlg_data->descr_lb = gtk_label_new(if_info->description);
+        else
+          if_dlg_data->descr_lb = gtk_label_new("");
+      }
       gtk_misc_set_alignment(GTK_MISC(if_dlg_data->descr_lb), 0.0, 0.5);
       gtk_table_attach_defaults(GTK_TABLE(if_tb), if_dlg_data->descr_lb, 2, 3, row, row+1);
 
@@ -663,8 +645,6 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
       gtk_table_attach_defaults(GTK_TABLE(if_tb), if_dlg_data->details_bt, 8, 9, row, row+1);
 #endif
 
-      open_if(if_info->name, if_dlg_data);
-
       if_data = g_list_append(if_data, if_dlg_data);
 
       row++;
@@ -702,7 +682,7 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
   gtk_widget_grab_default(close_bt);
 
   SIGNAL_CONNECT(cap_if_w, "delete_event", window_delete_event_cb, NULL);
-  SIGNAL_CONNECT(cap_if_w, "destroy", capture_if_destroy_cb, NULL);
+  SIGNAL_CONNECT(cap_if_w, "destroy", capture_if_destroy_cb, sc);
 
   gtk_widget_show_all(cap_if_w);
   window_present(cap_if_w);
@@ -710,7 +690,7 @@ capture_if_cb(GtkWidget *w _U_, gpointer d _U_)
   set_capture_if_dialog_for_capture_in_progress(is_capture_in_progress());
 
     /* update the interface list every 1000ms */
-  timer_id = gtk_timeout_add(1000, update_all, if_data);
+  timer_id = gtk_timeout_add(1000, update_all, sc);
 }
 
 
