@@ -2,7 +2,7 @@
  * Routines for Dynamic Service Change Response dissection
  * Copyright 2002, Anand V. Narwani <anand[AT]narwani.org>
  *
- * $Id: packet-dscrsp.c 23088 2007-10-07 23:46:44Z richardv $
+ * $Id: packet-dscrsp.c 32662 2010-05-04 21:44:03Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -70,7 +70,7 @@ dissect_dscrsp (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
       col_clear (pinfo->cinfo, COL_INFO);
       col_add_fstr (pinfo->cinfo, COL_INFO,
 		    "Dynamic Service Change Response ID = %u (%s)", transid,
-		    val_to_str (response, docsis_conf_code, "%s"));
+		    val_to_str (response, docsis_conf_code, "%d"));
     }
 
   if (tree)
